#!/usr/bin/env node
'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _winston = require('winston');

var _winston2 = _interopRequireDefault(_winston);

var _commander = require('commander');

var _commander2 = _interopRequireDefault(_commander);

var _nodeUserDefaults = require('node-user-defaults');

var _nodeUserDefaults2 = _interopRequireDefault(_nodeUserDefaults);

var _ticToc = require('tic-toc');

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _spinner = require('./spinner');

var _spinner2 = _interopRequireDefault(_spinner);

var _index = require('./index');

var _index2 = _interopRequireDefault(_index);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_nodeUserDefaults2.default.setLocation('~/.node-music-downloader');

_winston2.default.remove(_winston2.default.transports.Console);
_winston2.default.add(_winston2.default.transports.Console, { showLevel: false });

var pkg = _path2.default.resolve(__dirname, '../package.json');
var conf = JSON.parse(_fs2.default.readFileSync(pkg, 'utf8'));

_commander2.default.version(conf.version).usage('<options> query').option('-d, --debug', 'Enable debug mode').option('-b --bulk [file]', 'Bulk file, one query per line').option('-f, --force [url]', 'Youtube url to use').option('-o, --output [output]', 'Output directory').option('-t, --token [token]', 'Youtube authentication token').option('-r, --results [results]', 'Max results to check', parseInt).option('-s, --save-token', 'Store the Youtube authentication token for future use').parse(process.argv);

// PROCESS ALL THE OPTIONS

if (_commander2.default.debug) {
  _winston2.default.level = 'debug';
}

var token = _commander2.default.token || _nodeUserDefaults2.default.read('token');
if (_commander2.default.saveToken) {
  _nodeUserDefaults2.default.write('token', token);
}
if (!token) {
  _winston2.default.error(_chalk2.default.red.bold('No token provided!'));
  process.exit();
}

if (_commander2.default.bulk) {
  var location = _path2.default.resolve(process.cwd(), _commander2.default.bulk);
  if (!_fs2.default.existsSync(location)) {
    _winston2.default.error(_chalk2.default.red.bold('Bulk file does not exist!'));
    process.exit();
  }
} else if (_commander2.default.args.length < 1) {
  _winston2.default.error(_chalk2.default.red.bold('No query provided!'));
  process.exit();
}

var queries = [];

if (_commander2.default.bulk) {
  var _location = _path2.default.resolve(process.cwd(), _commander2.default.bulk);
  queries = _fs2.default.readFileSync(_location, 'utf8').split('\n');
} else {
  queries = [_commander2.default.args.join(' ')];
}

var options = {
  directory: _commander2.default.output || process.cwd(),
  debug: _commander2.default.debug,
  link: _commander2.default.force,
  verbose: true,
  token: token
};

var allOptions = queries.filter(function (q) {
  return !!q;
}).map(function (query) {
  return _extends({}, options, { query: query });
});

// DOWNLOAD STARTS FROM HERE. EVENTS WILL BE EMITTED.

var downloadSingleOptions = function downloadSingleOptions(args) {
  return new _bluebird2.default(function (resolve) {
    var db = args.debug;
    var vb = args.verbose;
    if (!db && vb) {
      _spinner2.default.start();
    }

    var downloader = (0, _index2.default)(args);
    downloader.on('log', function () {
      if (db) {
        _winston2.default.debug.apply(_winston2.default, arguments);
      }
    });
    downloader.on('warning', function () {
      if (!db && vb) {
        _spinner2.default.warn.apply(_spinner2.default, arguments);
      }
    });
    downloader.on('updateState', function () {
      if (!db && vb) {
        _spinner2.default.text.apply(_spinner2.default, arguments);
      }
    });

    downloader.on('success', function () {
      if (db) {
        return _winston2.default.debug('Completed in ' + Math.round((0, _ticToc.toc)() * 100) / 100 + 's.');
      }
      if (!vb) {
        return undefined;
      }
      var time = Math.round((0, _ticToc.toc)() * 100) / 100;
      _spinner2.default.succeed('Download of ' + _chalk2.default.bold(args.query) + ' complete in ' + _chalk2.default.bold(time + 's') + '!');
      return resolve();
    });

    downloader.on('fail', function (error) {
      if (db) {
        return _winston2.default.error('Failed with error: ' + error.message);
      }
      if (vb) {
        return error.message.split('\n').map(function (e) {
          return _spinner2.default.fail(e);
        });
      }
      return undefined;
    });
  });
};

_bluebird2.default.map(allOptions, function (opt) {
  return downloadSingleOptions(opt);
}, { concurrency: 1 });